﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Dynamic;
using System.Globalization;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Data;

namespace ReproduceInfragisticsIssue
{
    /// <summary>
    /// Class to serve as base class for MVVM View Models in WPF
    /// </summary>
    public class ViewModelBase : INotifyPropertyChanged
    {
        /// <summary>
        /// Public event to fire when a property changes
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged = null;

        /// <summary>
        /// Raises a property changed event
        /// </summary>
        /// <param name="property">The name of the property.  If not specified, it will take the name of the property it is called from</param>
        public void RaisePropertyChanged([CallerMemberName] String property = null)
        {
            if (this.PropertyChanged != null)
                this.PropertyChanged(this, new PropertyChangedEventArgs(property));
        }
    }

    /// <summary>
    /// Bindable dynamic dictionary taken from http://stackoverflow.com/questions/14171098/binding-datagrid-to-observablecollectiondictionary
    /// </summary>
    public class BindableDynamicDictionary : DynamicObject, INotifyPropertyChanged
    {
        #region Members
        private Dictionary<String, Object> data = new Dictionary<String, Object>();
        #endregion

        #region Selected Property
        private Boolean selected = true;

        /// <summary>
        /// Gets or set whether the object is selected
        /// </summary>
        public Boolean Selected
        {
            get
            {
                return (this.selected);
            }
            set
            {
                this.selected = value;
                this.RaisePropertyChanged();
            }
        }
        #endregion

        #region Indexer Property
        /// <summary>
        /// You can still use this as a dictionary.
        /// </summary>
        /// <param name="key">The key to look up</param>
        /// <returns>The object associated with the key</returns>
        public Object this[String key]
        {
            get
            {
                if (this.data.ContainsKey(key))
                    return (this.data[key]);
                return (null);
            }
            set
            {
                this.data[key] = value;
                this.RaisePropertyChanged(key);
            }
        }
        #endregion

        #region Construction
        /// <summary>
        /// Creates a new BindableDynamicDictionary with an empty internal dictionary.
        /// </summary>
        public BindableDynamicDictionary()
        {
        }

        /// <summary>
        /// Copies the contents of the given dictionary to initilize the internal dictionary.
        /// </summary>
        /// <param name="data">The data to create the object with</param>
        public BindableDynamicDictionary(IDictionary<String, Object> data)
        {
            this.data = new Dictionary<String, Object>(data);
        }

        /// <summary>
        /// Makes a clone of the current object
        /// </summary>
        /// <returns>A new object</returns>
        public BindableDynamicDictionary Clone()
        {
            BindableDynamicDictionary clone = new BindableDynamicDictionary();
            foreach (KeyValuePair<String, Object> entry in this.data)
                clone[entry.Key] = entry.Value;
            return (clone);
        }

        /// <summary>
        /// Makes a new object using the same schema as the current object
        /// </summary>
        /// <returns>A new object with empty fields</returns>
        public BindableDynamicDictionary EmptyCopy()
        {
            BindableDynamicDictionary clone = new BindableDynamicDictionary();
            foreach (KeyValuePair<String, Object> entry in this.data)
                clone[entry.Key] = String.Empty;
            return (clone);
        }
        #endregion

        #region Methods
        /// <summary>
        /// This allows you to get properties dynamically.
        /// </summary>
        /// <param name="binder"></param>
        /// <param name="result"></param>
        /// <returns></returns>
        public override Boolean TryGetMember(GetMemberBinder binder, out Object result)
        {
            return (this.data.TryGetValue(binder.Name, out result));
        }

        /// <summary>
        /// This allows you to set properties dynamically.
        /// </summary>
        /// <param name="binder"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public override Boolean TrySetMember(SetMemberBinder binder, Object value)
        {
            this.data[binder.Name] = value;
            this.RaisePropertyChanged(binder.Name);
            return (true);
        }

        /// <summary>
        /// This is used to list the current dynamic members.
        /// </summary>
        /// <returns></returns>
        public override IEnumerable<String> GetDynamicMemberNames()
        {
            return (this.data.Keys);
        }

        /// <summary>
        /// Helper to see if key exists in dictionary
        /// </summary>
        /// <param name="key"></param>
        /// <returns>bool</returns>
        public Boolean ContainsKey(String key)
        {
            return (this.data.ContainsKey(key));
        }

        /// <summary>
        /// Removes a value using the specified key
        /// </summary>
        /// <param name="key">The key to remove</param>
        public void Remove(String key)
        {
            this.data.Remove(key);
        }

        /// <summary>
        /// Changes a key name
        /// </summary>
        /// <param name="oldkey">The old key name</param>
        /// <param name="newkey">The new key name</param>
        public void ChangeKey(String oldkey, String newkey)
        {
            // we make a copy of the data into a new dictionary to maintain the order of the keys
            Dictionary<String, Object> newdata = new Dictionary<String, Object>();
            foreach (String key in this.data.Keys)
            {
                if (key == oldkey)
                    newdata[newkey] = this.data[key];
                else
                    newdata[key] = this.data[key];
            }
            this.data = newdata;
        }
        #endregion

        #region INotifyPropertyChanged Members
        /// <summary>
        /// Public event to fire when a property changes
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged = null;

        /// <summary>
        /// Raises a property changed event
        /// </summary>
        /// <param name="property">The name of the property.  If not specified, it will take the name of the property it is called from</param>
        public void RaisePropertyChanged([CallerMemberName] String property = null)
        {
            if (this.PropertyChanged != null)
                this.PropertyChanged(this, new PropertyChangedEventArgs(property));
        }
        #endregion
    }

    class MyViewModel : ViewModelBase
    {
        public MyViewModel()
        {
            BindableDynamicDictionary bdd = null;

            for (int i = 0; i < 10; i++)
            {
                bdd = new BindableDynamicDictionary();

                bdd["Label"] = $"row {i+1}";

                for (int j = 0; j < 5; j++)
                {
                    bdd[$"Att {j + 1}"] = "some value";
                }

                options.Add(bdd);
            }
        }


        private ObservableCollection<BindableDynamicDictionary> options = new ObservableCollection<BindableDynamicDictionary>();
        public ObservableCollection<BindableDynamicDictionary> Options
        {
            get
            {
                return (this.options);
            }
            set
            {
                this.options = value;
                base.RaisePropertyChanged();
            }
        }

        private ObservableCollection<LevelInfoItem> dropdownvalues = null;
        public ObservableCollection<LevelInfoItem> DropdownValues
        {
            get
            {
                return (this.dropdownvalues);
            }
            set
            {
                this.dropdownvalues = value;
                base.RaisePropertyChanged();
            }
        }

        private String dropdowndisplaymemberpath = null;
        public String DropDownDisplayMemberPath
        {
            get
            {
                return (this.dropdowndisplaymemberpath);
            }
            set
            {
                this.dropdowndisplaymemberpath = value;
                base.RaisePropertyChanged();
            }
        }
    }

    public class LevelInfoItem
    {
        /// <summary>
        /// Gets or sets the level label
        /// </summary>
        public String Label { get; set; }

        /// <summary>
        /// Gets or sets the level value
        /// </summary>
        public String Value { get; set; }

    }

    internal class ValueToDropDownValueConverter : IMultiValueConverter
    {
        private ObservableCollection<LevelInfoItem> allItems;
        #region IMultiValueConverter Members
        public Object Convert(Object[] values, Type targetType, Object parameter, CultureInfo culture)
        {
            String item = (String)values[0];
            if (values[1] is String)
                return (item);
            allItems = (ObservableCollection<LevelInfoItem>)values[1];
            if (allItems == null)
                return (item);
            LevelInfoItem foundItem = allItems.SingleOrDefault(v => v.Value == item);
            return (foundItem == null ? null : foundItem.Value);
        }

        public Object[] ConvertBack(Object value, Type[] targetTypes, Object parameter, CultureInfo culture)
        {
            LevelInfoItem item = (LevelInfoItem)value;
            if (item == null)
                return (new object[] { null, allItems });
            return (new object[] { item.Value, allItems });
        }
        #endregion
    }

    internal class DropDownValueToValueConverter : IMultiValueConverter
    {
        private ObservableCollection<LevelInfoItem> allItems;
        #region IMultiValueConverter Members
        public Object Convert(Object[] values, Type targetType, Object parameter, CultureInfo culture)
        {
            String item = (String)values[0];
            allItems = (ObservableCollection<LevelInfoItem>)values[1];
            if (allItems == null)
                return (null);
            //if (item == null)
            //{
            //	LevelInfoItem newItem = allItems[allItems.Count - 1];
            //	newItem.IsContinuous = allItems[0].IsContinuous;
            //	return newItem; // newly added item
            //}
            LevelInfoItem oItem = allItems.FirstOrDefault(v => v.Value == item);
            return (oItem);
        }

        public Object[] ConvertBack(Object value, Type[] targetTypes, Object parameter, CultureInfo culture)
        {
            LevelInfoItem item = (LevelInfoItem)value;
            if (item == null)
            {
                return (new object[] { null, allItems });
            }

            String val = item.Value;
            return (new object[] { val, allItems });
        }
        #endregion
    }

    internal class CountZeroVisibilityConverter : IValueConverter
    {
        #region IValueConverter Members
        public Object Convert(Object value, Type targetType, Object parameter, CultureInfo culture)
        {
            if ((Int32)value == 0)
                return ("Visible");
            return ("Collapsed");
        }

        public Object ConvertBack(Object value, Type targetType, Object parameter, CultureInfo culture)
        {
            throw (new NotImplementedException());
        }
        #endregion
    }

}
